"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_p_1 = require("fs-extra-p");
const path = require("path");
const bluebird_lst_1 = require("bluebird-lst");
const ts = require("typescript");
const JsDocGenerator_1 = require("./JsDocGenerator");
const __awaiter = require("./awaiter");
const filenameToMid = (function () {
    if (path.sep === '/') {
        return (it) => it;
    }
    else {
        const separatorExpression = new RegExp(path.sep.replace('\\', '\\\\'), 'g');
        return (it) => it.replace(separatorExpression, '/');
    }
})();
function generateDeclarationFile(moduleName, declarationFiles, compilerOptions, out, basePath, mainFile) {
    return __awaiter(this, void 0, void 0, function* () {
        console.log(`Generating d.ts to ${out}`);
        const fileNameToModuleId = {};
        const relativeOutDir = path.relative(basePath, compilerOptions.outDir);
        yield fs_extra_p_1.mkdirs(path.dirname(out));
        const eol = "\n";
        const indent = "  ";
        const output = fs_extra_p_1.createWriteStream(out, { mode: parseInt('644', 8) });
        return yield new bluebird_lst_1.default((resolve, reject) => {
            output.on("finish", resolve);
            output.on("error", reject);
            for (const sourceFile of declarationFiles) {
                writeDeclaration(sourceFile, compilerOptions, relativeOutDir);
            }
            output.end();
        });
        function writeDeclaration(declarationFile, compilerOptions, relativeOutDir) {
            if (declarationFile.text.length === 0) {
                return;
            }
            let sourceModuleId;
            let baseName = "";
            const fileNameWithoutExt = declarationFile.fileName.slice(0, -5).replace(/\\/g, "/");
            const name = fileNameWithoutExt.substring(compilerOptions.outDir.length + 1);
            if (moduleName) {
                baseName = moduleName + '/';
                sourceModuleId = moduleName;
                if (name !== "index") {
                    sourceModuleId += '/' + relativeOutDir;
                }
            }
            else {
                sourceModuleId = relativeOutDir;
            }
            baseName += relativeOutDir;
            const parentDir = name.includes("/") ? `${baseName}/${path.dirname(name)}` : baseName;
            if (name !== "index") {
                sourceModuleId += '/' + name;
            }
            if (declarationFile.fileName.endsWith("main.d.ts") || (mainFile != null && `${fileNameWithoutExt}.js`.includes(mainFile))) {
                sourceModuleId = moduleName;
            }
            output.write(`declare module "${sourceModuleId}" {${eol}${indent}`);
            fileNameToModuleId[path.resolve(fileNameWithoutExt).replace(/\\/g, "/")] = sourceModuleId;
            const mainBasename = path.basename(mainFile, ".js");
            const content = JsDocGenerator_1.processTree(declarationFile, (node) => {
                if (node.kind === ts.SyntaxKind.ExternalModuleReference) {
                    const expression = node.expression;
                    if (expression.text.charAt(0) === '.') {
                        return ' require(\'' + filenameToMid(path.join(path.dirname(sourceModuleId), expression.text)) + '\')';
                    }
                }
                else if (node.kind === ts.SyntaxKind.DeclareKeyword) {
                    return '';
                }
                else if (node.kind === ts.SyntaxKind.StringLiteral && (node.parent.kind === ts.SyntaxKind.ExportDeclaration || node.parent.kind === ts.SyntaxKind.ImportDeclaration)) {
                    const text = node.text;
                    if (text.charAt(0) === '.') {
                        if (text.charAt(1) === '.') {
                            const m = fileNameToModuleId[path.resolve(path.dirname(declarationFile.fileName), text).replace(/\\/g, "/")];
                            if (m != null) {
                                return ` "${m}"`;
                            }
                            return ` "${baseName}/${text.substring(3)}"`;
                        }
                        else {
                            if (text.charAt(1) === '/' && text.substring(2) === mainBasename) {
                                return ` "${moduleName}"`;
                            }
                            return ` "${parentDir}/${text.substring(2)}"`;
                        }
                    }
                }
                return null;
            });
            let prev = content.replace(new RegExp(eol + '(?!' + eol + '|$)', 'g'), '$&' + indent);
            prev = prev.replace(/;/g, '');
            if (indent != "    ") {
                prev = prev.replace(/    /g, indent);
            }
            output.write(prev);
            if (prev.charAt(prev.length - 1) != '\n') {
                output.write(eol);
            }
            output.write('}' + eol + eol);
        }
    });
}
exports.generateDeclarationFile = generateDeclarationFile;
//# sourceMappingURL=declarationGenerator.js.map